package com.csc.library.dwr;

import java.sql.Statement;

import com.csc.library.database.CscConnection;
import com.csc.library.database.StaticProperties;
import com.csc.library.process.TransferTimeProcessRealTime;
import com.csc.library.session.DbInquiry;
import com.csc.library.session.DbRecord;
import com.csc.library.session.DbTable;
import com.csc.library.session.InitialInquiry;
import com.csc.library.session.InitialRecord;
import com.csc.library.session.InitialTable;
import com.csc.library.utilities.CscCalendar;
import com.csc.library.utilities.CscTime;
import com.csc.library.utilities.MyLog;

public class ReCalTtimecurrent1 extends DWR_base{

	private String emp ;
	private String[] shift ;
	private String[] times ;
	private String[] fee ;
	private String[] inp_param ;
	/* input shift : time0id, timein1, timeout1, timein2, timeout2, oldtime0 
	 * input times : dateid, time0id, m_dt_bg, m_tm_bg, m_dt_en, m_tm_en, runno
	 * input fee : night shift fee, transport fee
	 * input param : start date, end date, time0id, shift updated flag, time updated flag, in updated, out updated
	 * */
	public boolean calculate(String emp, String[] shift, String[] times, String[] fee, String[] param){
		DbRecord tcur = null;
		try {
			this.setUProfile();
			initparam(emp, shift,times,fee,param);
			processChgShift();
			processChgDataBeforCalFormula();
			processChgDataAfterCalFormula();
			
		}catch(Exception e){
			MyLog.error(this,e);
		}
		
		return true ;
	}
	
	private void initparam(String emp, String[] shift, String[] times, String[] fee, String[] param){
		this.emp = emp ;
		this.shift = shift ;
		this.times = times ;
		this.fee = fee ;
		this.inp_param = param ;
	}
	
	private boolean processChgShift(){
		try {
			if(this.inp_param[3].equals("true")){
				DbRecord db = new InitialRecord(this.up).getDbRecord("Mempl_time0trans");
				db.setColumn("*");
				/* find and delete current shift */
				db.set("employeeid", this.emp);
				db.set("companyid", (String) this.up.get("companyid"));
				db.set("start_date", this.times[0]);
				db.set("end_date", this.times[0]);
				db.set("time0id", this.shift[5]);	/* find current shift */
				if(db.search() == 1){
					db.delete();
				}
				
				/* replace with new shift */
				db.set("employeeid", this.emp);
				db.set("companyid", (String) this.up.get("companyid"));
				db.set("start_date", this.times[0]);
				db.set("end_date", this.times[0]);
				db.set("time0id", this.shift[0]);
				db.set("wf_ref_doc", "EMPVIEW");
				db.save();	
				
				TransferTimeProcessRealTime trl = new TransferTimeProcessRealTime(this.up
						, this.emp
						, this.times[0]
						, this.times[0]);
				trl.process();
				MyLog.debug("!! complete transfertime process with new shift !!");
			}
		}catch(Exception e){
			MyLog.error(this,e);
			return false ;
		}
		return true ;
	}
	
	private boolean processChgDataBeforCalFormula(){
		DbRecord tcur = null;
		try {
			if(this.inp_param[4].equals("true")){ /* update swipe in-out status */
				DbTable dt_tcu1 = new InitialTable(this.up).getDbTable("Ttime_current1");
				dt_tcu1.setColumn("*");
				
				tcur = new InitialRecord(this.up).getDbRecord("Ttime_current");
				tcur.setColumn("*");
				tcur.set("employeeid", emp);
				tcur.set("companyid", this.up.getCompanyid());
				tcur.set("dateid", new CscCalendar(times[0]).getYYYYMMDD());
				tcur.set("time0id", this.inp_param[2]);
				
				tcur.putChild(dt_tcu1);
				
				if(tcur.search() == 1){
					tcur.refreshChild();
					
					DbTable db = (DbTable) tcur.getChild("ttime_current1");
					
					db.first();
					do{
						DbRecord rec = (DbRecord) db.getCurrentRecordManager();
						if(rec.getString("runno").equals(times[6])){ 
							processChgSwipeTime(rec);
							
							break;
						}
					}while(db.eof());
					
					int status = tcur.save();
					saveCustom(status, tcur);
				}
			}
		}catch(Exception e){
			MyLog.error(this,e);
			return false ;
		}
		return true ;
	}
	
	private boolean processChgSwipeTime(DbRecord rec){
		try {
			/* edit swipetime */

			if(this.inp_param[5].equals("true")){
				rec.set("m_dt_bg", new CscCalendar(times[2]).getYYYYMMDD());
				rec.set("m_tm_bg", times[3]);
				rec.set("forget_in", "0");
				rec.set("warn00", "");
				rec.set("source_in","7");
			}
			
			if(this.inp_param[6].equals("true")){
				rec.set("m_dt_en", new CscCalendar(times[4]).getYYYYMMDD());
				rec.set("m_tm_en", times[5]);
				rec.set("forget_out", "0");
				rec.set("warn01", "");
				rec.set("source_out","7");
			}
		}catch(Exception e){
			MyLog.error(this,e);
			return false;
		}
		return true;
	}
	
	/* not implement yet */
	private boolean processChgDataAfterCalFormula(){
		return true ;
	}
	
	private boolean processChgFee(DbRecord rec){
		try {
			/* edit night shift fee */
			int f1 = 0;
			int f2 = 0;
			
			try{
				f1 = Integer.parseInt(fee[0]);
			}catch(Exception e){}
			
			try{
				f2 = Integer.parseInt(fee[1]);
			}catch(Exception e){}
			
			rec.set("unused6", f1);
			rec.set("unused9", f1);
			
			/* edit transportation fee */
			rec.set("unused7", f2);
			rec.set("unused10", f2);
		}catch(Exception e){
			MyLog.error(this,e);
			return false;
		}
		return true ;
	}
	
	private boolean saveCustom(int status, DbRecord db){
		String PROC_NAME = "BACKUP_TIMECURRENT_CUSTOM";
		try {
			if(status == 2){	/* save complete */
				CscCalendar cs1 = new CscCalendar(db.getString("dateid"));
				StringBuffer procedure = new StringBuffer();
				procedure.append("exec ").append(PROC_NAME).append(" '").append(db.getString("employeeid"))
					.append("','").append(cs1.getYYYYMMDD()).append("','").append(cs1.getYYYYMMDD()).append("'");
				
				execSQL(procedure.toString());
			}
		}catch(Exception e){
			MyLog.error(this,e);
			return false;
		}
		return true ;
	}
	
	/**  EXECUTE SQL  */
    public void execSQL(String _SQL) {
		CscConnection conn = null;
		Statement stmt = null;
		try {
			if(_SQL != null && !_SQL.equals("")){
				conn = (CscConnection) StaticProperties.myConnection.getConnection(this.up.get("dbname"));
				stmt = conn.createStatement();
				
				MyLog.debug("!! execute sql back up ttime_current custom !! : " + _SQL);
				stmt.executeUpdate(_SQL);
			}else{
				MyLog.debug("!! SQL is null or empty !!");
			}
        } catch (Exception e) {
        	MyLog.error("!! Error : can't exec this SQL [" + _SQL + "] !!" ,e);           
        } finally {
        	try {
	        	if(stmt != null) stmt.close();
	        	if(conn != null) conn.close();
        	} catch(Exception e){
        		MyLog.error("!! Error : Can't close statement or connection !!",e);
        	}
        }
    }
	
    /* temporary not used */
	private String getTimeResult(DbRecord tcur, String[] times){
		StringBuffer rtn = new StringBuffer();
		String deli = "#";
		try {
			DbTable db = (DbTable) tcur.getChild("ttime_current1");
			
			db.first();
			do{
				DbRecord rec = (DbRecord) db.getCurrentRecordManager();
				if(rec.getString("runno").equals(times[6])){ /* edit */
					rtn.append(rec.getString("lt")).append(deli);	/* late */
					rtn.append(rec.getString("m_lv")).append(deli);	/* absent & leave */
					rtn.append(rec.getString("eventgrp")).append(deli); /* type */
					rtn.append(rec.getString("ac_ot")).append(deli);
					rtn.append(rec.getString("ot1")).append(deli);
					rtn.append(rec.getString("ot5")).append(deli);
					rtn.append(rec.getString("ot2")).append(deli);
					rtn.append(rec.getString("ot3")).append(deli);
					rtn.append(rec.getString("hour_d")).append(deli);
					rtn.append(rec.getString("runno"));
					
					break ;
				}
			}while(db.eof());
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return rtn.toString();
	}
	
	/* temporary not used */
	private String getUpdateTotal(String emp, String[] param){
		StringBuffer rtn = new StringBuffer();
		String deli = "#";
		try {
			CscTime ct_late = new CscTime();
			CscTime ct_absent = new CscTime();
			CscTime ct_leave = new CscTime();
			CscTime ct_ot = new CscTime();
			CscTime ct_ot1 = new CscTime();
			CscTime ct_ot5 = new CscTime();
			CscTime ct_ot2 = new CscTime();
			CscTime ct_ot3 = new CscTime();
			CscTime ct_work = new CscTime();
			
			DbInquiry inq = new InitialInquiry(this.up).getDbInquiry("Ttime_current1");
			inq.setColumn("employeeid,companyid,dateid,eventgrp,hour_d,m_lv,ac_ot,lt,ot1,ot5,ot2,ot3");
			inq.setFilter("employeeid = '" + emp + "' and dateid between '" + new CscCalendar(param[0]).getYYYYMMDD() +
					"' and '" + new CscCalendar(param[1]).getYYYYMMDD() + "'");
			inq.refresh();
			while(inq.next()){
				ct_late.add(Float.parseFloat(inq.getString("lt")));
				
				if(inq.getString("eventgrp").equals("J")){
					ct_absent.add(Float.parseFloat(inq.getString("m_lv")));
				}else{
					ct_leave.add(Float.parseFloat(inq.getString("m_lv")));
				}
				
				ct_ot.add(Float.parseFloat(inq.getString("ac_ot")));
				ct_ot1.add(Float.parseFloat(inq.getString("ot1")));
				ct_ot5.add(Float.parseFloat(inq.getString("ot5")));
				ct_ot2.add(Float.parseFloat(inq.getString("ot2")));
				ct_ot3.add(Float.parseFloat(inq.getString("ot3")));
				ct_work.add(Float.parseFloat(inq.getString("hour_d")));
			}
			
			rtn.append(ct_late.getFloat()).append(deli);	/* late */
			rtn.append(ct_absent.getFloat()).append(deli);	/* absent */
			rtn.append(ct_leave.getFloat()).append(deli);	/* leave */
			rtn.append(ct_ot1.getFloat()).append(deli);	/* ot1 */
			rtn.append(ct_ot5.getFloat()).append(deli);	/* ot5 */
			rtn.append(ct_ot2.getFloat()).append(deli);	/* ot2 */
			rtn.append(ct_ot3.getFloat()).append(deli);	/* ot3 */
			rtn.append(ct_work.getFloat());	/* leave */
			
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return rtn.toString() ;
	}
}
