package com.csc.library.process;

import java.sql.ResultSet;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Vector;

import com.csc.library.database.CscConnection;
import com.csc.library.database.StaticConfig;
import com.csc.library.database.StaticProperties;
import com.csc.library.session.DbInquiry;
import com.csc.library.session.DbRecord;
import com.csc.library.session.InitialInquiry;
import com.csc.library.session.InitialRecord;
import com.csc.library.timeattendance.ManageEmpLeaveStat;
import com.csc.library.utilities.CheckNull;
import com.csc.library.utilities.CscCalendar;
import com.csc.library.utilities.MyLog;
import com.csc.library.utility.DHMConvert;

public class BackupEmpAnnualLeave extends MyProcess {
	
	String strAnnualCode = "";
	String startdate = "";
	String enddate = "";
	Vector vcData = new Vector();
	
	private String getAnnualCode(){
		return strAnnualCode;
	}
	
	public Vector getProcessList() {
		Vector v = new Vector();
		try {
			DbInquiry inq = (DbInquiry) new InitialInquiry(this.getUProfile()).getDbInquiry("MEMPLOYEE");
			inq.setParam("rsc_skip","true");
			inq.setColumn("employeeid");
			inq.setFilter("status in ('A','V')");
			inq.refresh();
			while(inq.next()){
				v.add(inq.getString("employeeid"));
			}			
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return v;
	}

	public void process() throws Exception {
		initBudgetYear();
		initAnnualLeaveCode();
		backup();
	}
	
	private void initBudgetYear(){
		try {
			startdate = (String) StaticConfig.getConfigCompany(this.upf, "TA1001");
			enddate = (String) StaticConfig.getConfigCompany(this.upf, "TA1002");
		}catch(Exception e){
			MyLog.error(this,e);
		}
	}
	
	private void backup(){
		try {
			Vector v = this.processList;
			
			for(int i=0; i < v.size(); i++){
				save(retrieveData((String)v.get(i)));
			}
		}catch(Exception e){
			MyLog.error(this,e);
		}
	}
	
	private String getRemain(String emp, String code){
		String rtn = "00:00:00";
		try {
			CscCalendar cur = new CscCalendar();
			ManageEmpLeaveStat mels = new ManageEmpLeaveStat(this.getUProfile(), emp, code);
            mels.setRequestDate(cur.getYYYYMMDD() , cur.getYYYYMMDD());
            rtn = mels.getLimitLeave(code);
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return rtn;
	}
	
	private void save(HashMap data){
		try {
			DHMConvert dhm = new DHMConvert();
			
			DbRecord db = (DbRecord) new InitialRecord(this.getUProfile()).getDbRecord("MEMPL_ANNUALLEAVE_HIS");
			db.set("employeeid", (String) data.get("employeeid"));
			db.set("companyid", (String) data.get("companyid"));
			db.set("vacation0", (String) data.get("vacation0"));
			db.set("vacation1", (String) data.get("vacation1"));
			db.set("leave_used", getLeaveUsed((String) data.get("leave_used"), (String) data.get("hour_d")));	
			db.set("remain", getRemain((String) data.get("employeeid"), strAnnualCode));
			db.set("start_date", (String) data.get("startdate"));		
			db.set("end_date", (String) data.get("enddate"));	
			db.set("lastupdate", new CscCalendar().getYYYYMMDD()); 
			db.save();
		}catch(Exception e){
			MyLog.error(this,e);
		}
	}
	
	private String getLeaveUsed(String lv, String hour_d){
		String rtn = "00:00:00";
		try {
			if(Double.parseDouble(lv) > 0){
				DHMConvert dhm = new DHMConvert();
				rtn = dhm.MinToDHM(dhm.timeToMin(Double.parseDouble(lv)), Double.parseDouble(hour_d));
			}
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return rtn;
	}
	
	// get data from memployee (vacation0,vacation1 and used from function) 
	private HashMap retrieveData(String empid){
		CscConnection conn = null;
		Statement stmt = null;
		ResultSet rs = null;
		
		HashMap hm = new HashMap();
		
		try {
			CheckNull chknull = new CheckNull();
			conn = (CscConnection) StaticProperties.myConnection.getConnection(upf.get("dbname"));
			stmt = conn.createStatement();
			rs = stmt.executeQuery(getQuery(empid));
			
			while(rs.next()){
				hm.put("employeeid", rs.getString("employeeid"));
				hm.put("companyid", rs.getString("companyid"));
				hm.put("vacation0", chknull.chkNullString(rs.getString("vacation0"),"00:00:00"));
				hm.put("vacation1", chknull.chkNullString(rs.getString("vacation1"),"00:00:00"));
				hm.put("leave_used", chknull.chkNullString(rs.getString("lv_used"),"0.00"));
				hm.put("startdate", new CscCalendar(this.startdate).getYYYYMMDD());
				hm.put("enddate", new CscCalendar(this.enddate).getYYYYMMDD());
				hm.put("hour_d", chknull.chkNullString(rs.getString("hour_d"),"8.00"));
			}
			
			
		}catch(Exception e){
			MyLog.error(this,e);
		}finally{
			try {
				if(rs != null) rs.close();
				if(stmt != null) stmt.close();
				if(conn != null) conn.close();
			}catch(Exception e){
				MyLog.error(this,e);
			}
		}
		return hm;
	}
	
	private String getQuery(String empid){
		StringBuffer sql = new StringBuffer();
		try {
			sql.append("select m.employeeid,m.companyid,m.vacation0,m.vacation1,m.time0,m0.hour_d,");
			sql.append("dbo.FC_GETLEAVEHOUR('").append(empid).append("','");
			sql.append(this.strAnnualCode).append("','").append(new CscCalendar(this.startdate).getYYYYMMDD()).append("','");
			sql.append(new CscCalendar(this.enddate).getYYYYMMDD()).append("') as lv_used ");
			sql.append(" from memployee m left join mtime0 m0 on m.time0 = m0.time0id ");
			sql.append(" where m.employeeid = '").append(empid).append("'");
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return sql.toString();
	}
	
	// get annual leave code (support only 1 row of annual leave)
	private void initAnnualLeaveCode(){
		try {
			DbInquiry inq = (DbInquiry) new InitialInquiry(this.getUProfile()).getDbInquiry("MEVENTGRP");
			inq.setColumn("eventgrpid,daytype");
			inq.setFilter("daytype = '7'");
			inq.refresh();
			while(inq.next()){
				strAnnualCode = inq.getString("eventgrpid");
			}
		}catch(Exception e){
			MyLog.error(this,e);
		}
	}
}
