package com.csc.library.process;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Properties;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.csc.library.database.StaticConfig;
import com.csc.library.mail.MyMail;
import com.csc.library.mail.SendMail;
import com.csc.library.session.DbInquiry;
import com.csc.library.session.DbRecord;
import com.csc.library.session.InitialInquiry;
import com.csc.library.session.InitialRecord;
import com.csc.library.utilities.CheckNull;
import com.csc.library.utilities.CscCalendar;
import com.csc.library.utilities.MyLog;
import com.csc.library.utilities.ThaiUtilities;
import com.csc.library.utilities.UProfile;

public class MailOperator {

	private Node eNode = null;
	private UProfile upf = null;
	private Properties prop = new Properties();			/*  */
	private Properties prop_eng = new Properties();		/* ѧ */
	
	private StringBuilder stbBody = new StringBuilder(); 			/* ֧ͤҡ template */
	private StringBuilder stbBody_eng = new StringBuilder(); 		/* ֧ͤҡ template */
	private StringBuilder stbWorkContent = new StringBuilder(); 	/* ʶҹС÷ӧҹ () */
	private StringBuilder stbWorkContent_eng = new StringBuilder(); /* ʶҹС÷ӧҹ  (ѧ) */
	private StringBuilder stbMailBody = new StringBuilder();
	
	private String mailfrom = "";
	private String mailto = "";
	private String sendtoname = "";
	private String sendtoname_eng = "";
	private String title = "";
	private String tbranch = "";
	private String ebranch = "";
	private int count = 0;
	
	private ThaiUtilities tu = new ThaiUtilities();
	
	public void init(UProfile upf,Node nNode){
		this.upf = upf;
		eNode = nNode;
	}
	
	/* ˹ͼѺ */
	private void setSendToName(){
		try {
			if (eNode.getNodeType() == Node.ELEMENT_NODE) {
				Element eElement = (Element) eNode;
				
				String fname = getTagValue("FNAME",eElement);
				String lname = getTagValue("LNAME",eElement);
				String efname = getTagValue("EFULLNAME",eElement);
				
				tbranch = getTagValue("TBRANCH",eElement);
				ebranch = getTagValue("EBRANCH",eElement);
				
				sendtoname = fname + " " + lname;	/* ˹ͼ觶֧ */
				sendtoname_eng = efname;
			}
		}catch(Exception e){
			MyLog.error(this,e);
		}
	}
	
	/* ˹Ң */
	private void setBranchName(){
		try {
			if (eNode.getNodeType() == Node.ELEMENT_NODE) {
				Element eElement = (Element) eNode;
				
				tbranch = getTagValue("TBRANCH",eElement);
				ebranch = getTagValue("EBRANCH",eElement);
			}
		}catch(Exception e){
			MyLog.error(this,e);
		}
	}
	
	private String replace(String org, String[] value){
		try {
			return(MessageFormat.format(org, value)); 			
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return "";
	}
	
	private String getEmployeeCode(){
		if (eNode.getNodeType() == Node.ELEMENT_NODE) {
			Element eElement = (Element) eNode;
			return getTagValue("EMPLOYEEID",eElement);
		}
		return "";
	}
	
	private String getWorkDate(){
		if (eNode.getNodeType() == Node.ELEMENT_NODE) {
			Element eElement = (Element) eNode;
			return getTagValue("DATEID",eElement);
		}
		return "";
	}
	
	/* ˹ʶҹС÷ӧҹ ͧѹ */
	private void setBodyContext(){
		try {
			if (eNode.getNodeType() == Node.ELEMENT_NODE) {
				Element eElement = (Element) eNode;
				
				NodeList nlList = eElement.getElementsByTagName("DATEINFO");
			    for(int i=0; i < nlList.getLength(); i++){
			    	Node nValue = (Node) nlList.item(i);
			    	Element sElement = (Element) nValue;
			    	
			    	if(!getTagValue("EVENTGRP", sElement).equals("H") && !getTagValue("EVENTGRP", sElement).equals("I")
			    		&& !getTagValue("EVENTGRP", sElement).equals("T")){
			    			count++;
			    			
			    			setAbsent(sElement);			/* 礢Ҵҹ */
					    	setLeave(sElement);				/* */
					    	setChangeHoliday(sElement);		/* ¹ѹش */
					    	setChangeShift(sElement);		/* ¹ */
					    	setOT(sElement);				/* ͷ */
			    	}
			    }
			    
		    	setMailBody();
		    	setMailBodyEng();
			}
		}catch(Exception e){
			MyLog.error(this,e);
		}
	}
	
	/* ʶҹС÷ӧҹ body */
	private void setMailBody(){
		try {
			String tmp = stbBody.toString();
			String[] val = { sendtoname, stbWorkContent.toString(), tbranch };
			
			stbBody = new StringBuilder();
			stbBody.append(this.replace(tmp, val));
			
		}catch(Exception e){
			MyLog.error(this,e);
		}
	}
	
	/* ʶҹС÷ӧҹ body */
	private void setMailBodyEng(){
		try {
			String tmp = stbBody_eng.toString();
			String[] val = { sendtoname_eng, getEmployeeCode(), stbWorkContent_eng.toString(), ebranch };
			
			stbBody_eng = new StringBuilder();
			stbBody_eng.append(this.replace(tmp, val));
			
		}catch(Exception e){
			MyLog.error(this,e);
		}
	}
	
	/* ҧ tag ͤ */
	private String genText(Element e,String txt){
		return "<TR><TD>&nbsp;</TD><TD CLASS=\"mail\" NOWRAP>" + getTagValue("DATEID",e) + "&nbsp;&nbsp;&nbsp;" + txt + "</TD></TR>";
	}
	
	/* ʶҹТҴҹ  */
	private void setAbsent(Element e){
		try {
			String eventgrp = getTagValue("EVENTGRP",e);
			
			if(eventgrp.equals("J")){
				String forget_in = getTagValue("WARN00",e);
				String forget_out = getTagValue("WARN01",e);
				String forget_inout = getTagValue("WARN02",e);
				String overlate = getTagValue("WARN06",e);
				String leavebefore = getTagValue("WARN05",e);
				String c_tm_bg = getTagValue("C_TM_BG",e);
				String c_tm_en = getTagValue("C_TM_EN",e);
				String m_lv = getTagValue("M_LV",e);
				String hour_s = getTagValue("HOUR_S",e);
				
				String tmp = "";
				String tmp_eng = "";
				
				
				
				if(forget_inout.length() > 0){	/* ٴ͡ */
					tmp = getPropData("case02");
					tmp_eng = getPropData_eng("case02");
				}else if(forget_in.length() > 0){
					String[] val = {c_tm_bg};
					
					tmp = getPropData("case00");
					tmp_eng = getPropData_eng("case00");
					
					tmp = this.replace(tmp, val);
					tmp_eng = this.replace(tmp_eng, val);
				}else if(forget_out.length() > 0){
					String[] val = {c_tm_en};
					
					tmp = getPropData("case01");
					tmp_eng = getPropData_eng("case01");
					
					tmp = this.replace(tmp, val);
					tmp_eng = this.replace(tmp_eng, val);
				}else if(overlate.length() > 0){
					String[] val = {m_lv};
					
					tmp = getPropData("case08");
					tmp_eng = getPropData_eng("case08");
					
					tmp = this.replace(tmp, val);
					tmp_eng = this.replace(tmp_eng, val);
				}else if(leavebefore.length() > 0){
					String[] val = {m_lv};
					
					tmp = getPropData("case09");
					tmp_eng = getPropData_eng("case09");
					
					tmp = this.replace(tmp, val);
					tmp_eng = this.replace(tmp_eng, val);
				}else{
					MyLog.error("!! Data is incorrect, message will be blank on [" + getEmployeeCode() + "/" + getTagValue("DATEID",e) + "]");
				}
				
				/* ӹǹ . */
				
				//tmp = this.replace(tmp, val);
				//tmp_eng = this.replace(tmp_eng, val);
				
				stbWorkContent.append(genText(e,tmp));			/* ͤ body */
				stbWorkContent_eng.append(genText(e,tmp_eng));	/* ͤ body */
			}
		}catch(Exception ex){
			MyLog.error(this,ex);
		}
	}
	
	/* ֧ prop */
	private String getPropData(String code){
		try {
			return tu.ASCII2Unicode(prop.getProperty(code));
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return "";
	}
	
	/* ֧ prop */
	private String getPropData_eng(String code){
		try {
			return prop_eng.getProperty(code);
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return "";
	}
	
	/*  */
	private void setLeave(Element e){
		try {
			String doctype = getTagValue("DOCTYPE",e);
			if(doctype.equals("A")){
				/* leave template */
				String tmp = "";
				String tmp_eng = "";

				String m_lv = getTagValue("M_LV",e);
				String hour_s = getTagValue("HOUR_S",e);
				
				DbRecord db = getLeaveType(getTagValue("EVENTGRP",e));
				
				if(m_lv.equals(hour_s)){	/* ѹ */
					String[] val = {db.getString("tdesc") ,getTagValue("DOCNO",e)};
					String[] val_e = {db.getString("edesc") ,getTagValue("DOCNO",e)};
					
					tmp = getPropData("case10");
					tmp_eng = getPropData_eng("case10");
					
					tmp = this.replace(tmp, val);
					tmp_eng = this.replace(tmp_eng, val_e);
				}else{
					tmp = getPropData("case04");
					tmp_eng = getPropData_eng("case04");
					
					String[] val = {db.getString("tdesc")
							,getTagValue("M_LV",e)
							,getTagValue("C_TM_BG",e)
							,getTagValue("C_TM_EN",e)
							,getTagValue("DOCNO",e)};
					
					String[] val_e = {db.getString("edesc")
							,getTagValue("M_LV",e)
							,getTagValue("C_TM_BG",e)
							,getTagValue("C_TM_EN",e)
							,getTagValue("DOCNO",e)};
					
					tmp = this.replace(tmp, val);
					tmp_eng = this.replace(tmp_eng, val_e);
				}
				
				stbWorkContent.append(genText(e,tmp));	/* ͤ body */
				stbWorkContent_eng.append(genText(e,tmp_eng));	/* ͤ body */
			}
		}catch(Exception ex){
			MyLog.error(this,ex);
		}
	}
	
	/* ͷ */
	private void setOT(Element e){
		try {
			String doctype = getTagValue("DOCTYPE",e);
			if(doctype.equals("OM") || doctype.equals("O") || doctype.equals("B")){
				String tmp = getPropData("case07");
				String tmp_eng = getPropData_eng("case07");
				
				Vector v = getOTData(e);
				for(int i=0; i < v.size(); i++){
					DbRecord db = (DbRecord) v.get(i);
					
					String[] val = {getTagValue("AP_OT",e)
							,db.getString("tm_bg")
							,db.getString("tm_en")
							,getTagValue("DOCNO",e)
						};
					
					tmp = this.replace(tmp, val);
					tmp_eng = this.replace(tmp_eng, val);

					stbWorkContent.append(genText(e,tmp));	/* ͤ body */
					stbWorkContent_eng.append(genText(e,tmp_eng));	/* ͤ body */
				}
			}
		}catch(Exception ex){
			MyLog.error(this,ex);
		}
	}
	
	/*  */
	private void setLate(Element e){
		try {
			String lt = getTagValue("LT",e);
			if(Float.parseFloat(lt) > 0){
				String tmp = getPropData("case03");
				String tmp_eng = getPropData_eng("case03");
				
				String[] val = { lt };
				
				tmp = this.replace(tmp, val);					/* ӹǹ .  */
				tmp_eng = this.replace(tmp_eng, val);					/* ӹǹ .  */
				
				stbWorkContent.append(genText(e,tmp));	/* ͤ body */
				stbWorkContent_eng.append(genText(e,tmp_eng));	/* ͤ body */
			}
		}catch(Exception ex){
			MyLog.error(this,ex);
		}
	}
	
	/* ¹ */
	private void setChangeShift(Element e){
		try {
			String doctype = getTagValue("DOCTYPE",e);
			if(doctype.equals("C")){
				String tmp = getPropData("case06");
				String tmp_eng = getPropData_eng("case06");
				
				DbRecord db = getChangeShiftData(getTagValue("DOCNO",e));
				if(db != null){
					String[] val = { db.getCscCalendar("old_shift").getDDMMYYYY()
							,db.getCscCalendar("new_shift").getDDMMYYYY()
							,getTagValue("DOCNO",e)
						};
					
					tmp = this.replace(tmp, val);
					tmp_eng = this.replace(tmp_eng, val);
					
					stbWorkContent.append(genText(e,tmp));	/* ͤ body */
					stbWorkContent_eng.append(genText(e,tmp_eng));	/* ͤ body */
				}
			}
		}catch(Exception ex){
			MyLog.error(this,ex);
		}
	}
	
	/* ¹ѹش */
	private void setChangeHoliday(Element e){
		try {
			String doctype = getTagValue("DOCTYPE",e);
			if(doctype.equals("CH")){
				String tmp = getPropData("case05");
				String tmp_eng = getPropData_eng("case05");
				
				DbRecord db = getChangeHolidayData(getTagValue("DOCNO",e));
				if(db != null){
					String[] val = { db.getCscCalendar("change_date1").getDDMMYYYY()
							,db.getCscCalendar("change_date").getDDMMYYYY()
							,getTagValue("DOCNO",e)
						};
					
					tmp = this.replace(tmp, val);
					tmp_eng = this.replace(tmp_eng, val);
					
					stbWorkContent.append(genText(e,tmp));	/* ͤ body */
					stbWorkContent_eng.append(genText(e,tmp_eng));	/* ͤ body */
				}
			}
		}catch(Exception ex){
			MyLog.error(this,ex);
		}
	}
	
	/* ˹Ѻ */
	private void setEMailTo(){
		try {
			this.mailto = getTagValue("EMAIL",(Element) eNode);
		}catch(Exception e){
			MyLog.error(this,e);
		}
	}
	
	/* ֧㺢ͷ */
	private Vector getOTData(Element e){
		Vector v = new Vector();
		try {
			DbInquiry inq = new InitialInquiry(this.upf).getDbInquiry("TADJTIME0");
			inq.setColumn("docid,employeeid,companyid,doctype,updtype,dt_bg,dt_en,tm_bg,tm_en");
			inq.setFilter("employeeid = '" + getTagValue("EMPLOYEEID",(Element)eNode) + "' and docdate = '" + getTagValue("DATEID",e) 
					+ "' and updtype = 'O' and docid = '" + getTagValue("DOCNO",e) + "'");
			inq.refresh();
			while(inq.next()){
				v.add(inq.getCurrentDbRecord().cloneRecord(false));
			}
		}catch(Exception ex){
			MyLog.error(this,ex);
		}
		return v;
	}
	
	/* ֧¹ѹش */
	private DbRecord getChangeHolidayData(String docid){
		DbRecord db = null;
		try {
			db = new InitialRecord(this.upf).getDbRecord("TCHHOLIDAY");
			db.setColumn("chholidayid,employeeid,companyid,change_date1,change_date2");
			db.set("chholidayid", docid);
			if(db.search() == 1){
				return db;
			}
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return db;
	}
	
	/* ֧¹ѹش */
	private DbRecord getChangeShiftData(String docid){
		DbRecord db = null;
		try {
			db = new InitialRecord(this.upf).getDbRecord("TCHANGESHIFT_NSTDA");
			db.setColumn("tchshiftid,employeeid,companyid,start_date,end_date,old_shift,new_shift");
			db.set("tchshiftid", docid);
			if(db.search() == 1){
				return db;
			}
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return db;
	}
	
	/* ֧ͻ */
	private DbRecord getLeaveType(String eventgrp){
		try {
			DbRecord db = new InitialRecord(this.upf).getDbRecord("MEVENTGRP");
			db.setColumn("eventgrpid,companyid,tdesc,edesc");
			db.set("eventgrpid", eventgrp);
			if(db.search() == 1){
				return db;
			}
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return null;
	}
	
	private void setMailProp(){
		try {
			setSendToName();
			//setBranchName();
			setBodyContext();
		}catch(Exception e){
			MyLog.error(this,e);
		}
	}
	
	public void sendmail(){
		try {
			readTemplate();
			readTemplateEng();
			loadCaseProp();
			loadCasePropEng();
			setMailProp();
			
			if(count > 0){
				setEMailFrom();
				setEMailTo();
				setMailSubject();
				setMailContent();
				
				if(mailto != null && mailto.length() > 0){
					SendMail sm = new SendMail("", mailfrom, mailto, "", title, stbMailBody.toString());
					sm.send();
				}
			}
		}catch(Exception e){
			MyLog.error(this,e);
		}
	}
	
	/* ͤ  ѧ */
	private void setMailContent(){
		try {
			String HTMLFORM = "<html><meta http-equiv=\"Content-Type\" content=\"text/html; charset=TIS620\" />";
			String SPACE = "<BR><BR>";
			String ENDFORM = "</html>";
			
			stbMailBody.append(HTMLFORM);
			stbMailBody.append(stbBody);
			stbMailBody.append(SPACE);
			stbMailBody.append(stbBody_eng);
			stbMailBody.append(ENDFORM);
		}catch(Exception e){
			MyLog.error(this,e);
		}
	}
	
	/* ҹ template  */
	private void readTemplate(){
		try {
			CheckNull chk = new CheckNull();
			String file = chk.chkNullString((String) StaticConfig.getConfigCompany(this.upf, "TATMPLATE"));
			
			if(file instanceof String){
				Reader read = new InputStreamReader(new FileInputStream(file), "UTF-8");
				BufferedReader reader = new BufferedReader(read);  
		        String line = null;  
		        
		        while ((line = reader.readLine()) != null) {  
		        	stbBody.append(line);  
		        }   
			}
			
		}catch(Exception e){
			MyLog.error(this,e);
		}
	}
	
	/* ҹ template  ѧ */
	private void readTemplateEng(){
		try {
			CheckNull chk = new CheckNull();
			String file = chk.chkNullString((String) StaticConfig.getConfigCompany(this.upf, "TATMPLATEG"));
			
			if(file instanceof String){
				Reader read = new InputStreamReader(new FileInputStream(file), "UTF-8");
				BufferedReader reader = new BufferedReader(read);  
		        String line = null;  
		        
		        while ((line = reader.readLine()) != null) {  
		        	stbBody_eng.append(line);  
		        }   
			}
			
		}catch(Exception e){
			MyLog.error(this,e);
		}
	}
	
	/* ˹ */
	private void setEMailFrom(){
		try {
			CheckNull chk = new CheckNull();
			String email = chk.chkNullString((String) StaticConfig.getConfigCompany(this.upf, "TAEMAILFR"));
			
			if(email instanceof String){
				this.mailfrom = email;
			}
			
		}catch(Exception e){
			MyLog.error(this,e);
		}
	}
	
	/* ˹ title */
	private void setMailSubject(){
		try {
			CheckNull chk = new CheckNull();
			String subject = chk.chkNullString((String) StaticConfig.getConfigCompany(this.upf, "TAMAILSB"));
			
			if(subject instanceof String){
				this.title = subject;
			}
			
		}catch(Exception e){
			MyLog.error(this,e);
		}
	}
	
	private String getTagValue(String sTag, Element eElement) {
		try {
			NodeList nlList = eElement.getElementsByTagName(sTag).item(0).getChildNodes();
			if(nlList != null){
				Node nValue = (Node) nlList.item(0);
				if(nValue != null){
					return nValue.getNodeValue();
				}
			}
		}catch(Exception e){
			MyLog.error("Can't get data from tag : " + sTag + " on employeeid : " + getTagValue("EMPLOYEEID", (Element)eNode));
			MyLog.error(this,e);
		}
		return "";
	}

	/* Ŵ working status */
	private void loadCaseProp(){
    	try {
    		CheckNull chk = new CheckNull();
			String file = chk.chkNullString((String) StaticConfig.getConfigCompany(this.upf, "TACASE"));
			
			prop = new Properties();
    		prop.load(new FileInputStream(file));
    	} catch (IOException ex) {
    		ex.printStackTrace();
        }
	}
	
	/* Ŵ working status */
	private void loadCasePropEng(){
    	try {
    		CheckNull chk = new CheckNull();
			String file = chk.chkNullString((String) StaticConfig.getConfigCompany(this.upf, "TACASENG"));
			
			prop_eng = new Properties();
    		prop_eng.load(new FileInputStream(file));
    	} catch (IOException ex) {
    		ex.printStackTrace();
        }
	}
}
