package com.csc.library.service;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.DecimalFormat;
import java.util.Hashtable;

import std.library.DataTable;

import com.csc.library.database.CscConnection;
import com.csc.library.database.StaticConfig;
import com.csc.library.database.StaticProperties;
import com.csc.library.formula.FormulaString;
import com.csc.library.session.DbInquiry;
import com.csc.library.session.DbRecord;
import com.csc.library.session.InitialInquiry;
import com.csc.library.utilities.CscCalendar;
import com.csc.library.utilities.CscTime;
import com.csc.library.utilities.MyLog;
import com.csc.library.utilities.MyTreeMap;
import com.csc.library.utilities.UProfile;

public class ServiceTimeWorking extends TA_Service  {

	private DbInquiry inqAvgWorking = null;
	private DecimalFormat df = new DecimalFormat("#,##0.00");
	private CscConnection conn = null;
	private Statement stmt =null;
	private ResultSet rs = null;
	private float hourPerDay = 8.0f;
	private boolean isGetHourPerDay =false;
	
	public ServiceTimeWorking() {		
	}
	
	public ServiceTimeWorking(UProfile up) {
		this.up = up;
	}
	
	private void chkInqAvgWorking() {
		try {
			if (this.inqAvgWorking==null) {
				this.inqAvgWorking = new InitialInquiry(this.getUProfile()).getDbInquiry();
				this.inqAvgWorking.setParam(this.param);
				this.inqAvgWorking.setSchemaName(this.up.getSchemaName());
				this.inqAvgWorking.initMyTable("TWORKING_SUMMARY","","");
				this.inqAvgWorking.setColumn("*");
			}
		} catch (Exception e) {
			MyLog.error(this, e);
		}
	}
	
	/*
	 * ӡúǡ .÷ӧҹ¢ͧѡҹ
	 *  filter Ѻ ҡ鹹èӹǹ.
	 * ӡ search 
	 */
	private double getWorkingAverage(String filter) {
		double avg = 0;
		try {
			int count = 0;
			if (this.inqAvgWorking==null) {
				this.chkInqAvgWorking();
			}
			this.inqAvgWorking.setFilter(filter);
			this.inqAvgWorking.refresh();
			while (this.inqAvgWorking.next()) {
				avg += this.inqAvgWorking.getCurrentDbRecord().getDouble("working_avg");
				count++;
			}
			if (count>0) {
				avg = avg/count;
			}
		} catch (Exception e) {
			MyLog.error(this, e);
		}		
		return Double.parseDouble(this.df.format(avg));
	}
	
	/*
	 * ӡҪ ӧҹµ͹
	 *  CscCalendar  觨й filter ͹лյ
	 */
	public double getWorkingAverage(String empID, CscCalendar csc) {
		String filter = "employeeid='"+empID+"' and monthid="+Integer.parseInt(csc.getMM())+
			" and yearid="+csc.getYear();
		return this.getWorkingAverage(filter);
	}
	
	/*
	 * ӡҪ ӧҹµ͹
	 *  CscCalendar 繪ǧѹ 觨й filter ͹лյ
	 */
	public double getWorkingAverage(String empID, CscCalendar csc1, CscCalendar csc2) {
		String filter = "employeeid='"+empID+"'" +
			" and (monthid>="+Integer.parseInt(csc1.getMM())+" and monthid<="+Integer.parseInt(csc2.getMM())+") "+
			" and (yearid>="+csc1.getYear()+" and yearid<="+csc2.getYear()+")";
		return this.getWorkingAverage(filter);		
	}
	
	/*
	 * ӡô֧šŧҢͧѡҹǧҷ
	 * ѹ ѹŧ˹ҧ
	 */
	public MyTreeMap getSwipeData(String emp, CscCalendar cs1, CscCalendar cs2, String filter) {
		MyTreeMap tr = new MyTreeMap();
		try {
			int i=0;
			CscCalendar tmp = null;
			String filterCon = "employeeid='"+emp+"' and (swipedate between '"+cs1.getYYYYMMDD()+"' and '"+cs2.getYYYYMMDD()+"')";
			if (!filter.equals("")){
				filterCon += " and "+filter;
			}
			MyTreeMap detail = new MyTreeMap();
			DbInquiry inq = new InitialInquiry(this.up).getDbInquiry();
			inq.setSchemaName(this.up.getSchemaName());
			inq.initMyTable("Ttimetemp", filterCon, "swipedate,swipetime");
			inq.setColumn("employeeid, swipedate, swipetime, companyid");
			inq.refresh();
			while (inq.next()) {
				DbRecord rec = inq.getCurrentDbRecord();
				if (tmp == null) {
					detail = new MyTreeMap(); //红šŧѹ
					tmp = rec.getCscCalendar("swipedate");
					detail.put(Double.valueOf(rec.getString("swipetime")), Double.valueOf(rec.getString("swipetime")));
				} else if (!tmp.equalsDate(rec.getCscCalendar("swipedate"))) { //ѹ¹ ҧ treemap  红šŧ
					tr.put(tmp.getDDMMYYYY(), detail);
					detail = new MyTreeMap(); //红šŧѹ
					detail.put(Double.valueOf(rec.getString("swipetime")), Double.valueOf(rec.getString("swipetime")));
					tmp = rec.getCscCalendar("swipedate");
				} else { //ѹӢšŧŧ treemap 
					detail.put(Double.valueOf(rec.getString("swipetime")), Double.valueOf(rec.getString("swipetime")));
				}	
				i++;
			} 
			if (i!=0) {
				tr.put(tmp.getDDMMYYYY(), detail);
			}
		} catch (Exception e) {
			MyLog.error(this, e);
		}
		return tr;
	}
	 
	public float getLeaveRemain(String employeeid, String lv_type) {
		CscTime c = new CscTime();
		try {	
			String limit = ""; 
			FormulaString fs = new FormulaString();
			float hour_d = 8;
			String day_type = "";
			
			// Find data from MEMPLOYEE
			DbInquiry inqEmp = new InitialInquiry(this.getUProfile()).getDbInquiry();
			inqEmp.initMyTable("MEMPLOYEE_TA", "", "");
			inqEmp.setColumn("employeeid,companyid,vacation0,vacation1,time0");
			inqEmp.setFilter("employeeid='" + employeeid + "'");
			inqEmp.refresh();
			inqEmp.next();
			
			//Find hour_d from MTIME0
			try {
				DbInquiry inqTime = new InitialInquiry(this.getUProfile()).getDbInquiry();
				inqTime.initMyTable("MTIME0", "", "");
				inqTime.setColumn("time0id,companyid,hour_d");
				inqTime.setFilter("time0id='" + inqEmp.getString("time0") + "'");
				inqTime.refresh();
				inqTime.next();
				hour_d = inqTime.getCurrentDbRecord().getFloat("hour_d");
			} catch (Exception e) {
				MyLog.error(this, e);
			}
			
			if(hour_d<=0){
				this.hourPerDay =8.0f;
			}else{
				this.hourPerDay = hour_d;
			}
 			
			if(this.isGetHourPerDay){
				return 0.0f;
			}
			
			// Find limit from MEVENTGRP
			DbInquiry inq = new InitialInquiry(this.getUProfile()).getDbInquiry();
			inq.setSchemaName(this.getUProfile().getSchemaName());
			inq.initMyTable("MEVENTGRP","eventgrpid='"+lv_type+"'","");			
			inq.setColumn("*");
			inq.referLangOn();
			inq.refresh();
			if (inq.next()) {
				day_type = inq.getString("daytype");
				if (day_type.equalsIgnoreCase("7")) { //Ҿѡ͹ ԷҨҡѡҹ				
					try {						
						int i = fs.DhmToM(inqEmp.getString("vacation0"), hour_d) + fs.DhmToM(inqEmp.getString("vacation1"), hour_d);
						limit = fs.MTODhm(i, hour_d);
					} catch (Exception e) {
						MyLog.error(this, e);
						limit = "00:00:00";					
					}
				} else {
					if (inq.getString("limit").length()==1) { // ԷҨҡ
						limit = "0"+inq.getString("limit")+":00:00";
					} else {
						limit = inq.getString("limit")+":00:00";
					}
				}
			}
			// ---- End Find limit from event ----
						
			CscCalendar cscLeaveStart = new CscCalendar((String) StaticConfig.getConfigCompany(this.up, "ta1001"));
			CscCalendar cscLeaveEnd = new CscCalendar((String) StaticConfig.getConfigCompany(this.up, "ta1002"));
			CscCalendar cscVacationStart = new CscCalendar((String) StaticConfig.getConfigCompany(this.up, "ta1003"));
			CscCalendar cscVacationEnd = new CscCalendar((String) StaticConfig.getConfigCompany(this.up, "ta1004"));
			
			// ֧šҵա ҹҨҡ MCONFIG
							
			int minute_used = 0;
			
			/**
			 * Author : Nuiss
			 * Date : 14/07/2552
			 * Logic : ¡ա method  
			 *  getLeaveUse();
			String filter = "employeeid='"+employeeid+"' and lv_type='"+lv_type+"' ";
			if (day_type.equalsIgnoreCase("7")) {
				filter += "and (dateid between '"+cscVacationStart.getYYYYMMDD()+"' and '"+cscVacationEnd.getYYYYMMDD()+"')";
			} else {
				filter += "and (dateid between '"+cscLeaveStart.getYYYYMMDD()+"' and '"+cscLeaveEnd.getYYYYMMDD()+"')";
			}

			DbInquiry inqLeave = new InitialInquiry(this.getUProfile()).getDbInquiry();
			inqLeave.initMyTable("tleave_summary", filter, "");
			inqLeave.setColumn("*");
			inqLeave.refresh();			
			while (inqLeave.next()) {
				DbRecord rec = inqLeave.getCurrentDbRecord();
				minute_used +=  fs.Tm_To_Min(rec.getDouble("m_lv"));				
			}
			 */
			if (day_type.equalsIgnoreCase("7")) {
				minute_used = this.getLeaveUse(employeeid,cscVacationStart.getYYYYMMDD()
						  ,cscVacationEnd.getYYYYMMDD(),lv_type);
			}else{
				minute_used = this.getLeaveUse(employeeid,cscLeaveStart.getYYYYMMDD()
						  ,cscLeaveEnd.getYYYYMMDD(),lv_type);
			}//end if 
			int minute_limit = fs.DhmToM(limit, hour_d); 
			int minute_remain = minute_limit - minute_used;
			
			//System.out.println("== limit : "+limit+" > "+minute_limit);
			//System.out.println("== used : > "+minute_used);
			//System.out.println("== remain : > "+minute_remain);
						
			c.addMinute(minute_remain);
			//System.out.println("== Time Remain ====> "+c.getFloat());			
		} catch (Exception e) {
			MyLog.error(this, e);
		}
		return c.getFloat();
	}
	/**
	 * Author : Nuiss
	 * Date : 14/07/2552
	 * Logic : Ѻ  ͧҡ
	 * @return 
	 * 	-ͧҡ return int type 
	 */
	private int getLeaveUse(String employeeid,String lvBeginDate
									   ,String lvEndDate,String lvType){
		
 		String whereClause =  " employeeid='"+employeeid+"' and lv_type='"+lvType+"' "+
 										  " and (dateid between '"+lvBeginDate+"' and '"+lvEndDate+"')";
		int leaveMinuteUsed = 0;
		FormulaString fs = new FormulaString();
		try{
			DbInquiry inqLeave = new InitialInquiry(this.getUProfile()).getDbInquiry();
			inqLeave.initMyTable("tleave_summary", whereClause, "");
			inqLeave.setColumn("*");
			inqLeave.refresh();			
			while (inqLeave.next()) {
				DbRecord rec = inqLeave.getCurrentDbRecord();
				leaveMinuteUsed +=  fs.Tm_To_Min(rec.getDouble("m_lv"));				
			}
		}catch(Exception ex){
			System.out.print(">>> Error at ServiceTimeWorking.getLeaveUse()...");
		}//end try
		return leaveMinuteUsed;
	}//end method getLeaveUse
	
	public float getTimeLeaveUse(String employeeid,String lvBeginDate,String lvEndDate,String lvType){	
		
		CscCalendar cscLeaveStart = new CscCalendar((String) StaticConfig.getConfigCompany(this.up, "ta1001"));
		CscCalendar cscLeaveEnd = new CscCalendar((String) StaticConfig.getConfigCompany(this.up, "ta1002"));
		
		int minute_used = 0;
		if(lvBeginDate.equals("")||lvEndDate.equals("")){
			minute_used = this.getLeaveUse(employeeid,cscLeaveStart.getYYYYMMDD() ,cscLeaveEnd.getYYYYMMDD(),lvType);
		}else{
			minute_used = this.getLeaveUse(employeeid,new CscCalendar(lvBeginDate).getYYYYMMDD() ,new CscCalendar(lvEndDate).getYYYYMMDD(),lvType);
		}
		CscTime timeUse = new CscTime();
		timeUse.addMinute(minute_used);
		
		return timeUse.getFloat();
	}//end method getTimeLeaveUse
	
	public float getTimeLate(String employeeid,String lateStDate,String lateEndDate){
		CscTime timeLate = new CscTime();
		DataTable dtbLate = getTimeLate2DataTable(employeeid,lateStDate,lateEndDate);
		int lateMinute = 0;
		FormulaString fs = new FormulaString();
		
		for (int row=1;row<=dtbLate.rowCount();row++) {
			String[] ltTime = dtbLate.getItemString(row,"M_LV").replace(".", ":").split(":");
			String hour = "0";
			String minute = "00";
			if(ltTime.length==2){
				hour = (ltTime[0].equals("")?"0":ltTime[0]);
				minute = ((ltTime[1].length()==1||ltTime[1].equals(""))?"0"+ltTime[1]:ltTime[1]);
			}else{
				hour = (ltTime[0].equals("")?"0":ltTime[0]);;
			}//end if 	
			/*lateMinute +=  fs.Tm_To_Min(Double.valueOf(hour+"."+minute));*/		
			lateMinute +=  fs.Tm_To_Min(dtbLate.getItemDouble(row, "M_LV"));		
		}//end for

		timeLate.addMinute(lateMinute);

		return timeLate.getFloat();
	}//end method getTimeLate
	
	private DataTable getTimeLate2DataTable(String employeeid,String lateStDate,String lateEndDate){ 		
		/** 
		 Requirement has changed..all status need to be retrieved from tleave_summary only
		 ( previous version always retrieved from ttime_current1 and ttime_history1 )
		 Modified on 24-08-2010 by Kom.
		 **/
		StringBuffer currentWhereCluase = new StringBuffer();
		currentWhereCluase.append("employeeid ='").append(employeeid).append("' ");
		currentWhereCluase.append(" and dateid >='").append(lateStDate).append("' ");
		currentWhereCluase.append(" and dateid <='").append(lateEndDate).append("' "); 
		
		StringBuffer sqlQuery = new StringBuffer();
		sqlQuery.append("SELECT employeeid,dateid,m_lv,lv_type as eventgrp,'current' as sourcedata ");
		sqlQuery.append(" FROM tleave_summary "); 
		sqlQuery.append(" WHERE lv_type = 'LT'");
		sqlQuery.append(" and ").append(currentWhereCluase);
		sqlQuery.append(" ORDER BY employeeid ASC,dateid ASC");
		
		DataTable dtbData = null;
		try{	
			dtbData = new DataTable(this.retrieveData(sqlQuery.toString()),true);
		}catch(Exception ex){
			MyLog.error("!! Error at ServiceTimeWorking.getTimeLate2DataTable()!!");
		}finally{		
			this.closeConnection();
		}//end try
		return dtbData;
	 }//end method getTimeLate2DataTable
	
	private ResultSet retrieveData(String sqlQuery){
		try{
			conn = (CscConnection) StaticProperties.myConnection.getConnection(this.up.get("dbname"));
			stmt = conn.createStatement();
 			rs = stmt.executeQuery(sqlQuery);	
		}catch(Exception ex){
			System.out.print(">>> Nuiss Debug : Error on ServiceTimeWorking.retrieveData()")	;
			ex.printStackTrace();
		}//end try
		return rs;
	}//end method retrieveData
	
	private void closeConnection(){
		try {
			rs.close();
			if(stmt!=null){
				stmt.close();
			}
			conn.close();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	/**================================================
	 * Author : Nuiss
	 * Date : 20/05/2553
	 * Logic : for HTML Report site ONESQA[.]
	 * {LEAVEDOC_REPORT.jsp}
	 */
	private Hashtable htnMeventgrp = null;
	
	//constructor for retrieve data from table meventgrp
	public ServiceTimeWorking(UProfile up,boolean isRetrieveMeventgrp) {
		this.up = up;
		if(isRetrieveMeventgrp){
			this.retrieveMeventgrp();
		}
	}
	
	private void retrieveMeventgrp(){
		this.htnMeventgrp = new Hashtable();
		try{
			DbInquiry inqMeventgrp = new InitialInquiry(this.getUProfile()).getDbInquiry();
			inqMeventgrp.initMyTable("meventgrp", "", "");
			inqMeventgrp.setColumn("EVENTGRPID,TDESC");
			inqMeventgrp.refresh();			
			while (inqMeventgrp.next()) {
				DbRecord rec = inqMeventgrp.getCurrentDbRecord();
				String key = rec.getString("EVENTGRPID");	
				String value = rec.getString("TDESC");	
				htnMeventgrp.put(key, value);
			}
		}catch(Exception ex){
			System.out.print(">>> Error at ServiceTimeWorking.retrieveMeventgrp()...");
		}//end try
	}
	
	public DataTable rerieveEmpEnvironment(String employeeid){
  		String sqlQuery ="SELECT mp.vacationcode,mp.vacation0,mp.vacation1 " +
 		",mp.bossid,bu2.tdesc as bu2TDESC" +
 		",(select mpf.tdesc+mb.fname+'  '+mb.lname from memployee mb left outer join mprefix mpf on (mb.emp_prefix=mpf.prefixid) " +
 		"	where mb.employeeid =mp.bossid ) as bossname " +
 		",(select mpos.tdesc from memployee mb left outer join mposition mpos on (mb.emp_position=mpos.positionid) " +
 		"	where mb.employeeid =mp.bossid) as bossposition " +
 		",'Ţ '+case when maddr.taddr is null or maddr.taddr='' then '-'else maddr.taddr end " +
 		"+'  ' +case when maddr.tmoo is null or maddr.tmoo=''  then '-' else maddr.tmoo end " +
 		"+'  '+case when maddr.tsoi is null or maddr.tsoi='' then '-'else maddr.tsoi end " +
 		"+'  '+case when maddr.troad is null or maddr.troad ='' then '-'else maddr.troad end " +
 		"+' Ӻ/ǧ '+case when maddr.tdistrict is null or maddr.tdistrict='' then '-'else maddr.tdistrict end " +
 		"+'<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" +
 		" /ࢵ '+case when maddr.amphur is null or maddr.amphur ='' then '-'else maddr.amphur end " +
 		"+ ' ѧѴ '+ case when " +
 		"(select p.long_tname from csczipcode z,mprovince p " + 
 		"	where z.province =p.provinceid " +
 		"	and z.zipcode =maddr.zipcode " +
 		"	and z.tamphur=maddr.amphur) is null or " +
 		"(select p.long_tname " +
 		"	from csczipcode z,mprovince p " + 
 		"	where z.province =p.provinceid " +
 		"	and z.zipcode =maddr.zipcode " +
 		"	and z.tamphur=maddr.amphur) ='' then '-' else " +
 		"(select p.long_tname " +
 		"	from csczipcode z,mprovince p " + 
 		"	where z.province =p.provinceid " +
 		"	and z.zipcode =maddr.zipcode " +
 		"	and z.tamphur=maddr.amphur) end " +  
 		"+' ɳ '+case when maddr.zipcode is null or maddr.zipcode=''  then '-'else maddr.zipcode end " +
 		"+'  Ѿ '+case when maddr.tel is null or  maddr.tel='' then '-'else maddr.tel end as empaddress" +
 		" FROM memployee mp left outer join mempl_addr maddr on (mp.employeeid=maddr.employeeid and maddr.addr_current=1) " +
 		" left outer join mbu2 bu2 on (mp.bu2=bu2.bu2id) " +
 		" WHERE mp.employeeid ='"+employeeid+"'";
 		
		DataTable dtbEmpEnv=null;
		try{	
			dtbEmpEnv = new DataTable(this.retrieveData(sqlQuery),true);
		}catch(Exception ex){
			System.out.print(">>> Nuiss Debug : Error at ServiceTimeWorking.rerieveEmpEnvironment()...");
		}finally{		
			this.closeConnection();
		}//end try
		return dtbEmpEnv;
	}
	
	public DataTable getLastLeave(String employeeid){
		CscCalendar cscLeaveStart = new CscCalendar((String) StaticConfig.getConfigCompany(this.up, "ta1001"));
		CscCalendar cscLeaveEnd = new CscCalendar((String) StaticConfig.getConfigCompany(this.up, "ta1002"));
		CscCalendar cscVacationStart = new CscCalendar((String) StaticConfig.getConfigCompany(this.up, "ta1003"));
		CscCalendar cscVacationEnd = new CscCalendar((String) StaticConfig.getConfigCompany(this.up, "ta1004"));
		
		CscCalendar stDateWhere = (cscLeaveStart.beforeDate(cscVacationStart)?cscLeaveStart:cscVacationStart);
		CscCalendar endDateWhere = (cscLeaveEnd.afterDate(cscVacationEnd)?cscLeaveStart:cscVacationEnd);

		StringBuffer sqlQuery = new StringBuffer(); 
		sqlQuery.append("SELECT top(1)tl.docid,ta.employeeid,ta.start_date,ta.end_date");
		sqlQuery.append(",tl.lv_type,me.tdesc as lv_desc "); 
		sqlQuery.append(",case when ta.leave_format =0 then ta.leave_day ");
		sqlQuery.append("		when ta.leave_format =2 or ta.leave_format=3 then 0.5 ");
		sqlQuery.append("		when ta.leave_format =1 then ta.leave_hour ");
		sqlQuery.append("		else 0 END as lastLeaveTime ");
		sqlQuery.append(" FROM tleave_summary tl,tabsent_nstda ta,meventgrp me "); 
		sqlQuery.append(" WHERE tl.docid=ta.absentid ");
		sqlQuery.append(" and tl.lv_type=me.eventgrpid ");
		sqlQuery.append(" and tl.doctype ='A' ");
		sqlQuery.append(" and tl.employeeid ='").append(employeeid).append("' ");
		sqlQuery.append(" and tl.lv_bg_date>='").append(stDateWhere.getYYYYMMDD()).append("'");
		sqlQuery.append(" and tl.lv_bg_date<='").append(endDateWhere.getYYYYMMDD()).append("'");
		sqlQuery.append(" ORDER BY tl.dateid DESC");
		DataTable dtbLastLeave=null;
		try{	
			dtbLastLeave = new DataTable(this.retrieveData(sqlQuery.toString()),true);
		}catch(Exception ex){
			MyLog.error(">>> Error at ServiceTimeWorking.getLastLeave()...");
		}finally{		
			this.closeConnection();
		}//end try
		return dtbLastLeave;
	}
	
	public String getLeaveDescription(String eventgrpId){
		String eventgrpTdesc = "";
		if(this.htnMeventgrp !=null){
			if(this.htnMeventgrp.containsKey(eventgrpId)){
				eventgrpTdesc =this.htnMeventgrp.get(eventgrpId).toString();
			}
		}
		return eventgrpTdesc;
	}
	
	public String getLeaveRemain_DDHHMM(String employeeid, String lv_type){
		FormulaString fs = new FormulaString();
		int MM = (int) this.getLeaveRemain(employeeid,lv_type)*60;
		String DD = fs.MTODhm(MM, (int)this.hourPerDay);
		return DD;
	}
	
	public String getTimeLeaveUse_DDHHMM(String employeeid,String lvBeginDate,String lvEndDate,String lvType){

		FormulaString fs = new FormulaString();
		int MM = (int) this.getTimeLeaveUse(employeeid,lvBeginDate,lvEndDate,lvType)*60;
		
		//invoke this method because get and set hour_d=this.hourPerDay only
		isGetHourPerDay =true;
		this.getLeaveRemain(employeeid,lvType);
		isGetHourPerDay =false;
		 
		String DDHHM = fs.MTODhm(MM, (int)this.hourPerDay);
		return DDHHM;
	}
	
	public String getLeaveCurrent_DDHHMM(String employeeid,String lvType
			,int leaveFormat,float leaveDay,float leaveHour){
		
		//invoke this method because get and set hour_d=this.hourPerDay only
		isGetHourPerDay =true;
		this.getLeaveRemain(employeeid,lvType);
		isGetHourPerDay =false;
		
		FormulaString fs = new FormulaString();

		int MM =0;
		String DDHHMM = "00:00:00";
		
		switch(leaveFormat){
			case 0://ѹ
				 MM = (int) ((leaveDay*this.hourPerDay)*60);
				 DDHHMM= fs.MTODhm(MM, (int)this.hourPerDay);
				 break;
			case 2://Ҥ
				MM = (int) ((0.5*this.hourPerDay)*60);
				DDHHMM= fs.MTODhm(MM, (int)this.hourPerDay);
				break;
			case 3://Ҥ觺
				MM = (int) ((0.5*this.hourPerDay)*60);
				DDHHMM= fs.MTODhm(MM, (int)this.hourPerDay);
				break;
			case 1:// .
				MM = (int) ((leaveHour*this.hourPerDay)*60);
				DDHHMM= fs.MTODhm(MM, (int)this.hourPerDay);
				break;
			default:
				DDHHMM="00:00:00";
				break;
		}//end switch
		return DDHHMM;
	}
	
	public String sumLeave_DDHHMM(String employeeid,String lvType,String oldLeave,String curLeave){
		String totalLeaveDDHHMM ="00:00:00";
		FormulaString fs = new FormulaString();
		
		//invoke this method because get and set hour_d=this.hourPerDay only
		isGetHourPerDay =true;
		this.getLeaveRemain(employeeid,lvType);
		isGetHourPerDay =false;
		
		float oldLeaveMM = fs.DhmToM(oldLeave, this.hourPerDay);
		float curLeaveMM = fs.DhmToM(curLeave, this.hourPerDay);
		float totalLeave = (float) fs.addtime(oldLeaveMM, curLeaveMM);
		totalLeaveDDHHMM= fs.MTODhm((int) totalLeave, (int)this.hourPerDay);
		return totalLeaveDDHHMM;
	}
	
	/**================================================*/
}//end class ServiceTimeWorking

