package com.csc.library.timeattendance;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.DecimalFormat;

import com.csc.library.database.CscConnection;
import com.csc.library.database.StaticConfig;
import com.csc.library.database.StaticProperties;
import com.csc.library.session.DbRecord;
import com.csc.library.session.InitialRecord;
import com.csc.library.utilities.CscCalendar;
import com.csc.library.utilities.MyLog;
import com.csc.library.utilities.UProfile;

public class CalculateLeaveHour {
	
	String employeeid ;
	String date ;
	String docid ;
	UProfile upf ;
	CscCalendar csdate ;
	
	DbRecord leavedoc ;
	
	public CalculateLeaveHour(String empid, String date, String docid, UProfile upf){
		this.employeeid = empid;
		this.date = date;
		this.docid = docid ;
		this.csdate = new CscCalendar(date);
		this.upf = upf;
	}
	
	private boolean initLeaveDoc(){
		try {
			leavedoc = new InitialRecord(this.upf).getDbRecord("Tabsent_nstda");
			leavedoc.setColumn("*");
			leavedoc.set("absentid", docid);
			leavedoc.set("companyid", this.upf.getCompanyid());
			if(leavedoc.search() == 1){
				MyLog.debug("!! absentid found : " + this.docid);
				return true;
			}else{
				MyLog.error("!! Can not find leave document : " + this.docid);
				return false;
			}
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return false;
	}
	
	public float getLeaveHour(){
		try {
			if(initLeaveDoc()){
				if(leavedoc.getString("leave_format").equals("0")){	/* whole day */
					return getHourPerDay();
				}else if(leavedoc.getString("leave_format").equals("1")){	/* hours */
					return leavedoc.getFloat("leave_hour");
				}else if(leavedoc.getString("leave_format").equals("2") || leavedoc.getString("leave_format").equals("3")){ /* half day */
					return getHalfDayLeave();
				}else{
					MyLog.error("!! leave format is invalid on absentid : " + this.docid);
				}
			}
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return 0 ;
	}
	
	private float getHalfDayLeave(){
		try {
			if(isFixHourForHalfDay()){
				return getHourPerDay() / 2;
			}else{
				return getLeaveHourFromTadjtime0();
			}
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return 0;
	}
	
	private DbRecord getTime0Rec(String time0id){
		DbRecord time0 = null;
		try {
			time0 = new InitialRecord(this.upf).getDbRecord("Mtime0");
			time0.setColumn("time0id,companyid,status,hour_d");
			time0.set("time0id", time0id);
			time0.set("companyid", this.upf.getCompanyid());
			if(time0.search() == 1){
				return time0;
			}
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return time0;
	}
	
	private float getHourPerDay(){
		try {
			DbRecord emp = new InitialRecord(this.upf).getDbRecord("Memployee");
			emp.setParam("rsc_skip","true");
			emp.setColumn("employeeid,companyid,time0");
			emp.set("employeeid", this.employeeid);
			emp.set("companyid", this.upf.getCompanyid());
			if(emp.search() == 1){
				DbRecord time0 = getTime0Rec(emp.getString("time0"));
				
				if(time0 != null){
					if(time0.getString("status").equals("0")){
						return time0.getFloat("hour_d");
					}else{
						DbRecord time2 = new InitialRecord(this.upf).getDbRecord("Mtime2");
						time2.setColumn("*");
						time2.set("time0id", time0.getString("time0id"));
						time2.set("companyid", this.upf.getCompanyid());
						time2.set("yearid", csdate.getYear());
						time2.set("monthid", csdate.getMonth()+1);
						if(time2.search() == 1){
							DecimalFormat dc = new DecimalFormat("00");
							String sh1 = time2.getString("time" + dc.format(csdate.getDate()) + "1");	/* shift 1 */
							String sh2 = time2.getString("time" + dc.format(csdate.getDate()) + "2");	/* shift 2 */
							String sh3 = time2.getString("time" + dc.format(csdate.getDate()) + "3");	/* shift 3 */
							
							DbRecord curtime0 = null;
							if(!sh1.equals("")){
								curtime0 = getTime0Rec(sh1);
							}else if(!sh2.equals("")){
								curtime0 = getTime0Rec(sh2);
							}else if(!sh3.equals("")){
								curtime0 = getTime0Rec(sh3);
							}
							
							if(curtime0 != null){
								return curtime0.getFloat("hour_d");
							}else{
								return 0;
							}
						}
					}
				}
			}else{
				MyLog.error("!! Employee not found : " + this.employeeid);
			}
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return 0;
	}
	
	private float getLeaveHourFromTadjtime0() {
		CscConnection conn = null;
		Statement stmt =null;
		ResultSet resultSet = null;
		float leaveHour = 0.0f;
		
		try{
			String sqlQuery = " select employeeid,docid,lv from tadjtime0 where updtype='A' "+
			" and employeeid ='"+this.employeeid+"' "+
			" and docdate ='"+this.date+"' "+
			" and docid = '"+this.docid+"' ";
			
			conn = (CscConnection) StaticProperties.myConnection.getConnection(this.upf.get("dbname"));
			stmt = conn.createStatement();
			resultSet = stmt.executeQuery(sqlQuery);
			while(resultSet.next()){
				leaveHour = Float.valueOf(resultSet.getString("LV"));
			}//end while
		}catch(Exception ex){
			System.out.print("Error in Tadjtime0.getLeaveHourFromTadjtime0()...");
		}finally{
			try {
				if(resultSet != null) resultSet.close();
				if(stmt != null) stmt.close(); 
				if(conn != null) conn.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}//end try
		return leaveHour;
	}//end method getLeaveHourFromTadjtime0
	
	/* fix hour for request leave half day (1 : yes, 0 : no) */
	private boolean isFixHourForHalfDay(){
		try {
			String isfix = (String) StaticConfig.getConfigCompany(this.upf, "TAFXLV");
			if(isfix == null || isfix.equals("")){
				MyLog.error("!! TAFXLV in table mconfig is null or empty !!");
				return false;
			}else if(isfix.equalsIgnoreCase("1")){
				return true;
			}
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return false;
	}
	
	
}
